class = require 'hump.class'
if not love.filesystem.exists('maps') then
	love.filesystem.mkdir('maps')
end
maps = love.filesystem.enumerate('maps')


function love.load()
	love.filesystem.load 'ball.lua' ()
	love.filesystem.load 'enemy.lua' ()

	font = {}
	font.default = love.graphics.newFont(12)
	font.big     = love.graphics.newFont(24)
	font.huuuge  = love.graphics.newFont(96)
		
	love.physics.setMeter(64)
	
	world = {}
	world.width       = love.graphics.getWidth()
	world.height      = love.graphics.getHeight() - 128
	world.gravityX    = 0
	world.gravityY    = 200
	world.world = love.physics.newWorld(world.gravityX, world.gravityY)
	world.world:setCallbacks(beginContact, endContact, preSolve, postSolve)
	
	ground = {}
	ground.body    = love.physics.newBody(world.world, 0, world.height)
	ground.shape   = love.physics.newEdgeShape(0, 0, 2000, 0)
	ground.fixture = love.physics.newFixture(ground.body, ground.shape, 0)
	ground.shape   = ground.fixture:getShape()
	ground.fixture:setRestitution(0.8)
	ground.fixture:setFriction(0.8)
	
	ball = ball(32, world.height-32)
	
	objects = {}
	objects.enemies = {}
end

function love.update(dt)
	world.world:update(dt)
	for i, v in pairs(objects.enemies) do
		v:update(dt)
	end
end

function love.mousepressed(x, y, b)
	if b == 'l' then
		ball:launch(x, y)
	end
end

function love.keypressed(k, u)
	if tonumber(k) ~= 0 and (tonumber(k) or 10) <= #maps and tonumber(k) then
		love.load()
		love.filesystem.load('maps/map' .. tonumber(k) .. '.lua')()
	end
end

function love.draw()
	local width = love.graphics.getWidth()
	if ball:isStopped() then
		if #objects.enemies == 0 then
			displayGameOver('You win!', 'You destroyed all the enemies')
		else
			displayGameOver('You lose!', 'You didn\'t destroy all the enemies')
		end
	end
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(ball:getDistance(), 0, 0)
	ball:draw()
	for i, v in pairs(objects.enemies) do
		v:draw()
	end
	love.graphics.setColor(100, 50, 0)
	love.graphics.line(-4, world.height, world.width+4, world.height)
	for i = 1,9 do
		if love.filesystem.exists('maps/map'..i..'.lua') then
			love.graphics.setColor(255,255,255)
		else
			love.graphics.setColor(128,128,128)
		end
		love.graphics.print('map'..i..'.lua', width*(i-1)/9, 784)
	end
	love.graphics.setColor(255,255,255)
end

function beginContact(a, b, coll)
	local aBody=a:getBody()
	local bBody=b:getBody()
	local x1,y1=aBody:getLinearVelocity()
	local x2,y2=bBody:getLinearVelocity()
	local mass=aBody:getMass()
	if (x2^2+y2^2)>(x1^2+y1^2) then
		mass=bBody:getMass()
	end
	local force=((y2-y1)^2+(x2-x1)^2)*mass
	local aObject=a:getUserData()
	local bObject=b:getUserData()
	if type(aObject)=='table' then
		aObject:takeDamage(force)
	end
	if type(bObject)=='table' then
		bObject:takeDamage(force)
	end
end

function deleteObject(tab,obj)
    for i,v in ipairs(tab) do
        if v==obj then
            table.remove(tab,i)
            return
        end
    end
end

function displayGameOver(header,text)
	love.graphics.setFont(font.huuuge)
	love.graphics.setColor(128, 0, 0)
	love.graphics.printf(header,0,256,world.width,'center')
	love.graphics.setFont(font.default)
	love.graphics.printf(text,0,384,world.width,'center')
end